include "Share.inc"
include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext MonoPattern, ColorPattern, BulletPattern
ext VVramFront_, RowFlags_, ClearVVram_
ext HaltSubsys, ResumeSubsys

VramTop equ Vram+VramRowSize*1

dseg
Backup:
    defs VVramWidth*VVramHeight

zseg
CharColor:
    defb 0
CharCount:
    defb 0
CharCode:
    defb 0
plCount:
    defb 0    
yCount:
    defb 0
xCount:
byteCount:
    defb 0
flagByte:
bitCount:
    defb 0
colorBits:
    defb 0
maskByte:
invertedByte:
    defb 0
pVram:
    defw 0
pFlags:
    defw 0


cseg
InitVram: public InitVram
    clr <CharCode

    ldy #MonoPattern+CharHeight*Char_Ascii
    ldd #($07 shl 8) or (Char_Logo-Char_Ascii)
    bsr MakePatternMono

    ldy #MonoPattern+CharHeight*Char_Logo
    ldd #($05 shl 8) or (Char_Wall-Char_Logo)
    bsr MakePatternMono

    ldy #MonoPattern+CharHeight*Char_Wall
    ldd #($02 shl 8) or (Char_HardWall-Char_Wall)
    bsr MakePatternMono

    ldy #MonoPattern+CharHeight*Char_HardWall
    ldd #($07 shl 8) or (Char_Meter-Char_HardWall)
    bsr MakePatternMono

    ldy #MonoPattern+CharHeight*Char_Meter
    ldd #($06 shl 8) or (Char_Fence-Char_Meter)
    bsr MakePatternMono

    ldy #MonoPattern+CharHeight*Char_Fence
    ldd #($02 shl 8) or (Char_MyRobo-Char_Fence)
    bsr MakePatternMono

    ldy #ColorPattern
    ldb #Char_End-Char_MyRobo
    bsr MakePatternColor
rts

Wait:
    ldb #100
    do
        decb
    while ne | wend
rts

MakePatternMono:
    std <CharColor
    do
        jsr HaltSubsys
            lda <CharCode | sta ShareRam+Share_Char
            ldx #ShareRam+Share_Pattern
            lda <CharColor | sta <colorBits
            lda #PlaneCount | sta <plCount
            do
                pshs y
                    lda #CharHeight | sta <yCount
                    do
                        clrb
                        lda colorBits
                        bita #$01
                        if ne
                            orb ,y
                        endif
                        bita #$10
                        if ne
                            lda ,y
                            coma
                            sta <invertedByte
                            orb <invertedByte
                        endif
                        lda #VramStep | sta <byteCount
                        do
                            lda #8/VramStep | sta <bitCount
                            clra
                            do
                                lsla
                                lsla
                                lslb
                                if cs
                                    ora #$03
                                endif
                                dec <bitCount
                            while ne | wend
                            sta ,x+
                            dec <byteCount
                        while ne | wend
                        leay 1,y
                        dec <yCount
                    while ne | wend
                puls y
                lsr <colorBits
                dec <plCount
            while ne | wend
            lda #SubCommand_DefP | sta ShareRam+Share_Command
        jsr ResumeSubsys
        bsr Wait

        leay CharHeight,y
        inc <CharCode
        dec <CharCount
    while ne | wend
rts

MakePatternColor:
    stb <CharCount
    do
        jsr HaltSubsys
            lda <CharCode | sta ShareRam+Share_Char
            ldx #ShareRam+Share_Pattern
            lda #CharHeight*PlaneCount | sta <yCount
            do
                ldb ,y+
                lda #2
                sta <byteCount
                do
                    lda #4
                    sta <bitCount
                    clra
                    do
                        lsla
                        lsla
                        lslb
                        if cs
                            ora #$03
                        endif
                        dec <bitCount
                    while ne | wend
                    sta ,x+
                    dec <byteCount
                while ne | wend
                dec <yCount
            while ne | wend
            lda #SubCommand_DefP | sta ShareRam+Share_Command
        jsr ResumeSubsys
        bsr Wait

        inc <CharCode
        dec CharCount
    while ne | wend
rts


cseg
ClearRowFlags:
    pshs x
        ldx #RowFlags_
        do
            clr ,x+
            cmpx #RowFlags_+4
        while ne | wend
    puls x
rts

; void ClearScreen();
cseg
ClearScreen_:   public ClearScreen_
    pshs x,b,a
        jsr HaltSubsys
        lda #SubCommand_Clear
        sta ShareRam+Share_Command
        jsr ResumeSubsys

        ldx #Backup
        ldd #VVramWidth*VVramHeight
        do
            clr ,x+
            subd #1
        while ne | wend
        jsr ClearVVram_
        bsr ClearRowFlags
    puls x,b,a
rts


; word Put(word vram, byte c);
cseg
Put_: public Put_
    pshs a,b,x,y
        jsr HaltSubsys
            lda #SubCommand_PutP | sta ShareRam+Share_Command
            stx ShareRam+Share_Address
            stb ShareRam+Share_Char
        jsr ResumeSubsys
    puls a,b,x,y
    tfr x,d
    addd #VramStep
rts


VVramToVramRow: public VVramToVramRow
    lda #VVramWidth
    sta <xCount
    do 
        ldb ,x+
        cmpb ,y
        if ne
            stb ,y
            pshs x
                ldx <pVram
                bsr Put_
            puls x
        endif
        leay 1,y
        pshs x
            ldx <pVram
            leax 2,x
            stx <pVram
        puls x
        dec <xCount
    while ne | wend
    pshs x
        ldx <pVram
        leax VramRowSize-VVramWidth*2,x
        stx <pVram
    puls x
rts


; void VVramToVramAll();
VVramToVramAll_: public VVramToVramAll_
    pshs a,b,x,y
        ldx #VramTop
        stx <pVram
        ldx #VVramFront_
        ldy #Backup
        lda #VVramHeight
        sta <yCount
        do
            bsr VVramToVramRow
            dec <yCount
        while ne | wend
        bsr ClearRowFlags
    puls a,b,x,y
rts


; void VVramToVramChanged();
VVramToVramChanged_: public VVramToVramChanged_
    pshs a,b,x,y
        ldx #VramTop
        stx <pVram
        ldx #RowFlags_
        lda ,x+
        sta <flagByte
        stx <pFlags
        lda #1
        sta <maskByte

        ldx #VVramFront_
        ldy #Backup
        lda #VVramHeight
        sta <yCount
        do
            lda <flagByte
            anda <maskByte
            if ne
                bsr VVramToVramRow
            else
                leax VVramWidth,x
                leay VVramWidth,y
                pshs x
                    ldx <pVram
                    leax VramRowSize,x
                    stx <pVram
                puls x
            endif
            lsl <maskByte
            if eq
                pshs x
                    ldx <pFlags
                    lda ,x+
                    sta <flagByte
                    stx <pFlags
                    lda #1
                    sta <maskByte
                puls x
            endif
            dec <yCount
        while ne | wend
        bsr ClearRowFlags
    puls a,b,x,y
rts